// Pool Wx Monitoring

#include <Wire.h>
#include <SPI.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
// OneWire Dallas stuff from working h20F sketch
#include <OneWire.h>
#include <DallasTemperature.h>

// Data wire is connected to the NodeMCU ESP8266 digital pin 4 (D4)
#define ONE_WIRE_BUS 4
// Setup a oneWire instance to communicate with any OneWire devices
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature sensor
DallasTemperature sensors(&oneWire);

#define BME_SCK 13
#define BME_MISO 12
#define BME_MOSI 11
#define BME_CS 10

#define SEALEVELPRESSURE_HPA (1013.25)

// Defining the MQTT Topics to pub/sub
#define esp/Ta "Air Temp"
#define esp/Press "Pressure"
#define esp/Hum "Humidity"
#define esp/Dp "Dew Point"
#define esp/Hidx "Heat Index"
#define esp/Ts "Spa Temp"
#define esp/Tp "Pool Temp"

const char* ssid = "SSID";
const char* password =  "PASS";
const char* mqttServer = "192.168.1.113";
const int mqttPort = 1883;

WiFiClient espClient;
PubSubClient client(espClient);

Adafruit_BME280 bme; // I2C

unsigned long delayTime;

void setup() {
  Serial.begin(115200);
  Serial.println("BME280");

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("Connecting to WiFi..");
  }
  Serial.println("WiFi Connected+");

  client.setServer(mqttServer, mqttPort);
  client.setCallback(callback);

  while (!client.connected()) {
    Serial.println("Connecting to MQTT...");

    if (client.connect("ESP8266Client")) {

      Serial.println("MQTT Connected+");
      Serial.println();

    } else {

      Serial.print("failed with state ");
      Serial.print(client.state());
      delay(2000);

    }
  }

  client.publish("esp/test", "PoolWX Station");
  client.subscribe("esp/test");

  // Sensor Stuff ****************
  // Start up the One Wire library
  sensors.begin();
  bool status;

  // default settings
  status = bme.begin();
  if (!status) {
    Serial.println("Could not find a valid BME280 sensor, check wiring!");
    while (1);
  }

  Serial.println("-- Taking A Sip --");
  Serial.print("vvvvvvvvvvvvvvvvvv");
  // delayTime = 900000; // Not needed when using Deep Sleep

  Serial.println();

  delay(100); // let sensor boot up
}

void callback(char* topic, byte* payload, unsigned int length) {

  Serial.print("Message arrived in topic: ");
  Serial.println(topic);

  Serial.print("Message:");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }

  Serial.println();
  Serial.println("-----------------------");

}

void loop() {
  // Call sensors.requestTemperatures() to issue a global temperature and Requests to all devices on the bus
  sensors.requestTemperatures(); 
  printValues();
  client.loop();
  delay(delayTime);
}


void printValues() {
  Serial.print("Air Temp = ");
  int fTemp = bme.readTemperature() * 9.0 / 5.0 + 32;
  Serial.print(fTemp);
  Serial.println(" *F");
  //Serial.println(String(fTemp).c_str());
  client.publish("esp/Ta", String(fTemp).c_str(), true);
  client.subscribe("esp/Ta");

  Serial.print("Spa Temp = ");
  int sTemp = sensors.getTempFByIndex(0);
  Serial.print(sensors.getTempFByIndex(0));
  Serial.println(" *F");
  client.publish("esp/Ts", String(sTemp).c_str(), true);
  client.subscribe("esp/Ts");

  Serial.print("Pool Temp = ");
  int pTemp = sensors.getTempFByIndex(1);
  Serial.print(sensors.getTempFByIndex(1));
  Serial.println(" *F");
  client.publish("esp/Tp", String(pTemp).c_str(), true);
  client.subscribe("esp/Tp");

  Serial.print("Pressure = ");
  int bPress = bme.readPressure() / 100.0F / 33.9;
  Serial.print(bPress);
  Serial.println(" inHg");
  client.publish("esp/Press", String(bPress).c_str(), true);

  Serial.print("Humidity = ");
  int rH = bme.readHumidity();
  Serial.print(rH);
  Serial.println(" %");
  client.publish("esp/Hum", String(rH).c_str(), true);

  Serial.print("Dew Point = ");
  int dPoint = fTemp - ((100 - rH) / 5);
  Serial.print(dPoint);
  Serial.println(" *F");
  client.publish("esp/Dp", String(dPoint).c_str(), true);

  Serial.print("Heat Index = ");
  int hIndex = 0.5 * (fTemp + 61.0 + ((fTemp - 68.0) * 1.2) + (rH * 0.094));
  Serial.print(hIndex);
  Serial.println(" *F");
  client.publish("esp/Hidx", String(hIndex).c_str(), true);
  Serial.println("^^^^^^^^^^^^^^^^^^");

  delay(1000); // Delay so MQTT has time to publish

  Serial.println();
  Serial.println("I'm alive, but I'm going into deep sleep mode for a while");
  Serial.flush ();
  ESP.deepSleep(1.8e9); // 10e6=10 seconds, 3e8=5min, 6e8=10min, 9e8=15min, 1.8e9=30min -- divide minutes by 6e7

  delay(1000); // Delay so deep.Sleep has time

  Serial.println("You will never see this");
}
